switch_example:
	// %rdi = x, %rsi = y, %rdx = z

	// Make a backup of z, because division will overwrite %rdx.
	movq	%rdx, %rcx

	// If we're out of the range of the switch cases, go to
	// the default case.
	cmpq	$6, %rdi // compare x to 6
	ja	.L8      // (unsigned)x > 6   What if x was negative?

	// Now do an indirect jump ("*") through the jump table.
	// %rdi is the index into the jump table, 8 is the entry size
	jmp	*.L4(,%rdi,8)

	// switch to the read-only data section of the .o file
	.section	.rodata
	.align 8
.L4:	// the jump table holds a bunch of addresses (of code labels)
	.quad	.L8  // x = 0 (= default case)
	.quad	.L3  // x = 1
	.quad	.L5  // x = 2
	.quad	.L9  // x = 3
	.quad	.L8  // x = 4 (= default case)
	.quad	.L7  // x = 5
	.quad	.L7  // x = 6 (= case 5)

	// switch back to the text (code) section of the .o file
	.text 
.L3:    // case 1: w = y * z;
	movq	%rsi, %rax
	imulq	%rdx, %rax
	ret
.L5:    // case 2: w = y / z
	movq	%rsi, %rax
	// sign-extend %rax (64-bit) into %rdx:%rax (128-bit)
	cqto
	// divide by %rcx. Destination is implicitly the 128-bit %rdx:%rax
	idivq	%rcx
	jmp	.L6 // fall through
.L9:    // case 3: w = 1
	movl	$1, %eax
.L6:    // common part of cases 2 + 3: w += z  (not in the jump table)
	addq	%rcx, %rax
	ret
.L7:    // case 5: case 6: w = 1; w -= z
	movl	$1, %eax
	subq	%rdx, %rax
	ret
.L8:    // default: w = 2
	movl	$2, %eax
	ret


.LC0:
	.string	"switch_example(%ld,%ld,%ld) = %ld\n"
main:
	pushq	%rbx
	movl	$0, %ebx
	jmp	.L11
.L12:
	movl	$5, %edx
	movl	$20, %esi
	movq	%rbx, %rdi
	call	switch_example
	movq	%rax, %r9
	movl	$5, %r8d
	movl	$20, %ecx
	movq	%rbx, %rdx
	movl	$.LC0, %esi
	movl	$1, %edi
	movl	$0, %eax
	call	__printf_chk
	addq	$1, %rbx
.L11:
	cmpq	$9, %rbx
	jle	.L12
	movl	$0, %eax
	popq	%rbx
	ret
